"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskServiceComponent = void 0;
const tslib_1 = require("tslib");
// Copyright (c) 2023 Sourcefuse Technologies
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const bpmn_service_1 = require("@sourceloop/bpmn-service");
const core_2 = require("@sourceloop/core");
const controllers = tslib_1.__importStar(require("./controllers"));
const loopback4_authentication_1 = require("loopback4-authentication");
const passport_bearer_1 = require("loopback4-authentication/passport-bearer");
const repositories_1 = require("./repositories");
const event_workflow_mapping_repository_1 = require("./repositories/event-workflow-mapping.repository");
const services_1 = require("./services");
let TaskServiceComponent = class TaskServiceComponent {
    constructor(application) {
        this.application = application;
        this.providers = {};
        this.services = [
            services_1.CamundaService,
            services_1.EventProcessorService,
            services_1.EventQueueServiceSQS,
            services_1.HttpClientService,
            services_1.TaskOperationService,
            services_1.UtilityService,
            services_1.WorkflowOperationService,
            services_1.WebhookService,
            services_1.TaskDbService,
            services_1.ApiKeyVerificationService,
        ];
        this.bindings = [];
        this.application.component(core_2.CoreComponent);
        this.application.api({
            openapi: '3.0.0',
            info: {
                title: 'Task Service',
                version: '1.0.0',
            },
            paths: {},
            components: {
                securitySchemes: core_2.SECURITY_SCHEME_SPEC,
            },
            servers: [{ url: '/' }],
        });
        this.application.component(bpmn_service_1.WorkflowServiceComponent);
        this.controllers = [
            controllers.EventQueueController,
            controllers.TaskServiceController,
            controllers.ApiKeyController,
            controllers.EventsController,
        ];
        this.repositories = [
            repositories_1.EventRepository,
            repositories_1.TaskRepository,
            event_workflow_mapping_repository_1.EventWorkflowMappingRepository,
            repositories_1.TaskWorkFlowMappingRepository,
            repositories_1.WebhookSubscriptionsRepository,
            repositories_1.ApiKeyRepository,
        ];
    }
    /**
     * Setup ServiceSequence by default if no other sequnce provided
     */
    setupSequence() {
        this.application.sequence(core_2.ServiceSequence);
        this.application
            .bind(loopback4_authentication_1.Strategies.Passport.BEARER_STRATEGY_FACTORY.key)
            .toProvider(passport_bearer_1.BearerStrategyFactoryProvider);
        this.application
            .bind(loopback4_authentication_1.Strategies.Passport.BEARER_TOKEN_VERIFIER.key)
            .toProvider(passport_bearer_1.BearerTokenVerifyProvider);
    }
};
exports.TaskServiceComponent = TaskServiceComponent;
exports.TaskServiceComponent = TaskServiceComponent = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(core_1.CoreBindings.APPLICATION_INSTANCE)),
    tslib_1.__metadata("design:paramtypes", [rest_1.RestApplication])
], TaskServiceComponent);
//# sourceMappingURL=component.js.map