"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventQueueController = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const core_2 = require("@sourceloop/core");
const loopback4_authentication_1 = require("loopback4-authentication");
const loopback4_authorization_1 = require("loopback4-authorization");
const models_1 = require("../models");
const event_queue_service_1 = require("../services/event-queue.service");
const baseUrl = '/event-queue';
let EventQueueController = class EventQueueController {
    constructor(eventQueueService) {
        this.eventQueueService = eventQueueService;
    }
    async enqueueEvent(event) {
        try {
            await this.eventQueueService.enqueueEvent(event);
        }
        catch (error) {
            throw new rest_1.HttpErrors.InternalServerError('Failed to enqueue event');
        }
    }
    async healthCheck() {
        try {
            const healthCheckResponse = await this.eventQueueService.healthCheck();
            return healthCheckResponse;
        }
        catch (error) {
            throw new rest_1.HttpErrors.InternalServerError('Failed to perform health check');
        }
    }
};
exports.EventQueueController = EventQueueController;
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({ permissions: ["3" /* TaskPermssionKey.AddToQueue */] }),
    (0, rest_1.post)(`${baseUrl}/enqueue`, {
        responses: {
            [200 /* STATUS_CODE.OK */]: {
                description: 'Enque model instance',
                content: {
                    [core_2.CONTENT_TYPE.JSON]: { schema: (0, rest_1.getModelSchemaRef)(models_1.Event) },
                },
            },
            [204 /* STATUS_CODE.NO_CONTENT */]: {
                description: 'Event enqueued successfully',
            },
            [500 /* STATUS_CODE.INTERNAL_SERVER_ERROR */]: {
                description: 'Failed to enqueue event',
            },
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)({
        content: {
            [core_2.CONTENT_TYPE.JSON]: {
                schema: (0, rest_1.getModelSchemaRef)(models_1.Event, {
                    title: 'Event',
                }),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], EventQueueController.prototype, "enqueueEvent", null);
tslib_1.__decorate([
    (0, loopback4_authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.get)(`${baseUrl}/ping`, {
        responses: {
            [200 /* STATUS_CODE.OK */]: {
                description: 'Health check response',
                content: {
                    [core_2.CONTENT_TYPE.JSON]: {
                        schema: {
                            type: 'object',
                        },
                    },
                },
            },
            [500 /* STATUS_CODE.INTERNAL_SERVER_ERROR */]: {
                description: 'Failed to perform health check',
            },
        },
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", Promise)
], EventQueueController.prototype, "healthCheck", null);
exports.EventQueueController = EventQueueController = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)('services.EventQueueServiceSQS')),
    tslib_1.__metadata("design:paramtypes", [event_queue_service_1.EventQueueServiceSQS])
], EventQueueController);
//# sourceMappingURL=event-queue.controller.js.map