"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskServiceController = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const task_operation_service_1 = require("../services/task-operation.service");
const loopback4_authorization_1 = require("loopback4-authorization");
const webhook_service_1 = require("../services/webhook.service");
const loopback4_authentication_1 = require("loopback4-authentication");
const models_1 = require("../models");
const api_key_verification_service_1 = require("../services/api-key-verification.service");
const repository_1 = require("@loopback/repository");
const repositories_1 = require("../repositories");
const core_2 = require("@sourceloop/core");
const baseUrl = 'tasks';
let TaskServiceController = class TaskServiceController {
    constructor(taskOpsService, webhookService, apiKeyVerificationService, taskWorkflowMapping) {
        this.taskOpsService = taskOpsService;
        this.webhookService = webhookService;
        this.apiKeyVerificationService = apiKeyVerificationService;
        this.taskWorkflowMapping = taskWorkflowMapping;
    }
    async updateTask(tasks) {
        await this.taskOpsService.taskUpdateFlow(tasks.taskKey, tasks.payload);
    }
    async subscribeToWebhook(subscriber, request) {
        const apiKey = request.headers['x-api-key'];
        const apiSecret = request.headers['x-api-secret'];
        let isValidKeys = false;
        if (apiKey && apiSecret) {
            isValidKeys = await this.apiKeyVerificationService.verifyApiKeys(apiKey, apiSecret);
        }
        if (!isValidKeys) {
            throw new rest_1.HttpErrors.Unauthorized('Invalid API key or secret');
        }
        await this.webhookService.addToSubscription(subscriber.url, subscriber.key);
    }
    async mapTaskToWorkflow(taskWorkflowMapping) {
        await this.taskWorkflowMapping.create(taskWorkflowMapping);
    }
};
exports.TaskServiceController = TaskServiceController;
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)(`${baseUrl}`, { security: core_2.OPERATION_SECURITY_SPEC, responses: {} }),
    tslib_1.__param(0, (0, rest_1.requestBody)()),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [models_1.TaskDto]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskServiceController.prototype, "updateTask", null);
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)(`${baseUrl}/subscribe`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {},
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)()),
    tslib_1.__param(1, (0, core_1.inject)(rest_1.RestBindings.Http.REQUEST)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [models_1.SubscriberDTO, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskServiceController.prototype, "subscribeToWebhook", null);
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)(`${baseUrl}/mapping`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {},
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)({
        content: {
            [core_2.CONTENT_TYPE.JSON]: {
                schema: (0, rest_1.getModelSchemaRef)(models_1.TaskWorkflows, {
                    title: 'TaskWorkFlowMapping',
                }),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskServiceController.prototype, "mapTaskToWorkflow", null);
exports.TaskServiceController = TaskServiceController = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.service)(task_operation_service_1.TaskOperationService)),
    tslib_1.__param(1, (0, core_1.service)(webhook_service_1.WebhookService)),
    tslib_1.__param(2, (0, core_1.service)(api_key_verification_service_1.ApiKeyVerificationService)),
    tslib_1.__param(3, (0, repository_1.repository)(repositories_1.TaskWorkFlowMappingRepository)),
    tslib_1.__metadata("design:paramtypes", [task_operation_service_1.TaskOperationService,
        webhook_service_1.WebhookService,
        api_key_verification_service_1.ApiKeyVerificationService,
        repositories_1.TaskWorkFlowMappingRepository])
], TaskServiceController);
//# sourceMappingURL=task-service.controller.js.map