"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventProcessorService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const event_repository_1 = require("../repositories/event.repository");
const types_1 = require("../types");
const task_operation_service_1 = require("./task-operation.service");
const workflow_operation_service_1 = require("./workflow-operation.service");
const webhook_service_1 = require("./webhook.service");
let EventProcessorService = class EventProcessorService {
    constructor(eventRepo, taskOpsService, workflowOpsService, webhookService) {
        this.eventRepo = eventRepo;
        this.taskOpsService = taskOpsService;
        this.workflowOpsService = workflowOpsService;
        this.webhookService = webhookService;
    }
    async processEvent(event) {
        await this.eventRepo.create(event);
        const workflow = await this.workflowOpsService.execWorkflow(event.key, types_1.TaskServiceNames.EVENT);
        if (workflow) {
            // subscribe to tasks
            const url = await this.webhookService.getUrlOfSubscritption(event.key);
            if (url && event.payload.tasks) {
                const tasksPromises = event.payload.tasks.map((task) => this.webhookService.addToSubscription(url, task.key));
                await Promise.all(tasksPromises);
            }
            await this.taskOpsService.processTask(workflow.externalIdentifier, workflow.name, event.key, event.payload);
        }
        else {
            throw new rest_1.HttpErrors.NotFound('Workflow not found');
        }
    }
};
exports.EventProcessorService = EventProcessorService;
exports.EventProcessorService = EventProcessorService = tslib_1.__decorate([
    (0, core_1.injectable)({
        scope: core_1.BindingScope.TRANSIENT,
    }),
    tslib_1.__param(0, (0, repository_1.repository)(event_repository_1.EventRepository)),
    tslib_1.__param(1, (0, core_1.service)(task_operation_service_1.TaskOperationService)),
    tslib_1.__param(2, (0, core_1.service)(workflow_operation_service_1.WorkflowOperationService)),
    tslib_1.__param(3, (0, core_1.service)(webhook_service_1.WebhookService)),
    tslib_1.__metadata("design:paramtypes", [event_repository_1.EventRepository,
        task_operation_service_1.TaskOperationService,
        workflow_operation_service_1.WorkflowOperationService,
        webhook_service_1.WebhookService])
], EventProcessorService);
//# sourceMappingURL=event-processor.service.js.map