"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpClientService = void 0;
const tslib_1 = require("tslib");
// Copyright (c) 2023 Sourcefuse Technologies
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
let HttpClientService = class HttpClientService {
    get(url, options) {
        const processed = this.processOptions(url, options);
        return (0, node_fetch_1.default)(processed.url, {
            headers: processed.headers,
        }).then(res => this.handleRes(res));
    }
    delete(url, options) {
        const processed = this.processOptions(url, options);
        return (0, node_fetch_1.default)(processed.url, {
            method: 'delete',
            headers: processed.headers,
        }).then(res => this.handleRes(res));
    }
    post(url, body, options) {
        const processed = this.processOptions(url, options);
        const contentTypeHeader = 'content-type';
        return (0, node_fetch_1.default)(processed.url, {
            method: 'post',
            body: JSON.stringify(body),
            headers: {
                [contentTypeHeader]: 'application/json',
                ...processed.headers,
            },
        }).then(res => this.handleRes(res));
    }
    postFormData(url, body, options) {
        const processed = this.processOptions(url, options);
        return (0, node_fetch_1.default)(processed.url, {
            method: 'post',
            body: body,
            headers: {
                ...body.getHeaders(),
                ...processed.headers,
            },
        }).then(res => this.handleRes(res));
    }
    processOptions(url, options) {
        let headers = {};
        if (options === null || options === void 0 ? void 0 : options.query) {
            url = `${url}?${this.serialize(options.query)}`;
        }
        if (options === null || options === void 0 ? void 0 : options.urlParams) {
            for (const key in options.urlParams) {
                url = url.replace(new RegExp(`\\{${key}\\}`, 'gi'), options.urlParams[key]);
            }
        }
        if (options === null || options === void 0 ? void 0 : options.headers) {
            headers = options.headers;
        }
        return { url, headers };
    }
    async handleRes(res) {
        if (res.status === 200 /* STATUS_CODE.OK */) {
            return res.json().catch(e => Promise.resolve({}));
        }
        else if (res.status === 204 /* STATUS_CODE.NO_CONTENT */) {
            return Promise.resolve({});
        }
        else {
            throw new rest_1.HttpErrors.BadRequest(await res.text());
        }
    }
    serialize(obj) {
        const str = [];
        for (const p in obj) {
            if (obj.hasOwnProperty(p)) {
                str.push(`${encodeURIComponent(p)}=${encodeURIComponent(obj[p])}`);
            }
        }
        return str.join('&');
    }
};
exports.HttpClientService = HttpClientService;
exports.HttpClientService = HttpClientService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.TRANSIENT })
], HttpClientService);
//# sourceMappingURL=http.service.js.map