"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkflowOperationService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const bpmn_service_1 = require("@sourceloop/bpmn-service");
const types_1 = require("../types");
const repository_1 = require("@loopback/repository");
const repositories_1 = require("../repositories");
const camunda_service_1 = require("./camunda.service");
const rest_1 = require("@loopback/rest");
let WorkflowOperationService = class WorkflowOperationService {
    constructor(workflowRepo, eventWorkflowMapping, taskWorkflowMapping, camundaService) {
        this.workflowRepo = workflowRepo;
        this.eventWorkflowMapping = eventWorkflowMapping;
        this.taskWorkflowMapping = taskWorkflowMapping;
        this.camundaService = camundaService;
    }
    async execWorkflow(keyVal, taskOrEvent) {
        let taskOrWorkflowItem;
        switch (taskOrEvent) {
            case types_1.TaskServiceNames.TASK:
                taskOrWorkflowItem = await this.findTaskWorkflowByKey(keyVal);
                break;
            case types_1.TaskServiceNames.EVENT:
                taskOrWorkflowItem = await this._findEventWorkflowByKey(keyVal);
                break;
            default:
                break;
        }
        if (taskOrWorkflowItem) {
            const workflow = await this.findWorkflowByKey(taskOrWorkflowItem.workflowKey);
            if (workflow) {
                try {
                    await this.camundaService.execute(workflow.externalIdentifier, {});
                    return workflow;
                }
                catch (e) {
                    throw new rest_1.HttpErrors.NotFound('Workflow not found');
                }
            }
        }
        return null;
    }
    async findTaskWorkflowByKey(keyValue) {
        const filter = {
            where: {
                taskKey: keyValue,
            },
        };
        return this.taskWorkflowMapping.findOne(filter);
    }
    async _findEventWorkflowByKey(keyValue) {
        const filter = {
            where: {
                eventKey: keyValue,
            },
        };
        return this.eventWorkflowMapping.findOne(filter);
    }
    async findWorkflowByKey(keyValue) {
        const filter = {
            where: {
                name: keyValue,
            },
        };
        return this.workflowRepo.findOne(filter);
    }
};
exports.WorkflowOperationService = WorkflowOperationService;
exports.WorkflowOperationService = WorkflowOperationService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.TRANSIENT }),
    tslib_1.__param(0, (0, repository_1.repository)(bpmn_service_1.WorkflowRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(repositories_1.EventWorkflowMappingRepository)),
    tslib_1.__param(2, (0, repository_1.repository)(repositories_1.TaskWorkFlowMappingRepository)),
    tslib_1.__param(3, (0, core_1.service)(camunda_service_1.CamundaService)),
    tslib_1.__metadata("design:paramtypes", [bpmn_service_1.WorkflowRepository,
        repositories_1.EventWorkflowMappingRepository,
        repositories_1.TaskWorkFlowMappingRepository,
        camunda_service_1.CamundaService])
], WorkflowOperationService);
//# sourceMappingURL=workflow-operation.service.js.map